"{ Package: 'jv:libllvms' }"

"{ NameSpace: Smalltalk }"

LLVMObject subclass:#LLVMType
	instanceVariableNames:''
	classVariableNames:'Int1 Int16 Int32 Int64 IntPtr'
	poolDictionaries:'LLVMTypeKind'
	category:'LLVM-Core'
!

!LLVMType class methodsFor:'instance creation'!

function: argumentTypes returning: returnType
    ^ LLVM FunctionType: returnType  _: argumentTypes asLLVMObjectArray _: argumentTypes size _: 0

    "Created: / 07-07-2015 / 21:53:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 08-07-2015 / 22:58:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int1
    Int64 isNil ifTrue:[  
        Int64 := LLVM Int64Type
    ].
    ^ Int64

    "Created: / 07-07-2015 / 21:21:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 11-07-2015 / 07:01:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int16
    ^ LLVM Int16Type

    "Created: / 07-07-2015 / 21:21:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int32
    ^ LLVM Int32Type

    "Created: / 07-07-2015 / 21:21:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int64
    ^ LLVM Int64Type

    "Created: / 07-07-2015 / 21:21:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

int8
    ^ LLVM Int8Type

    "Created: / 07-07-2015 / 21:21:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

intptr
    IntPtr isNil ifTrue:[
        IntPtr := LLVM IntPtrType: LLVMTargetData new.  
    ].
    ^ IntPtr

    "Created: / 11-07-2015 / 07:05:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMType methodsFor:'accessing'!

kind
    ^ LLVM GetTypeKind: self

    "Created: / 07-07-2015 / 21:59:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMType methodsFor:'debugging-dumping'!

dumpOn: aStream
    aStream nextPutAll: self dumpString

    "Created: / 10-07-2015 / 14:32:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

dumpString
    | cstr str|

    cstr := LLVM PrintTypeToString: self.
    str := cstr copyCStringFromHeap.
    LLVM DisposeMessage: cstr.
    ^ str

    "Created: / 10-07-2015 / 14:31:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

printOn: aStream
    | kind |

    super printOn: aStream.
    aStream nextPutAll: self dumpString.
    aStream nextPutAll: ' ['.
    kind := self kind.
    kind == LLVMVoidTypeKind ifTrue:[ 
        aStream nextPutAll:'void'  
    ] ifFalse:[ kind == LLVMHalfTypeKind ifTrue:[ 
        aStream nextPutAll:'half'  
    ] ifFalse:[ kind == LLVMFloatTypeKind ifTrue:[ 
        aStream nextPutAll:'float'  
    ] ifFalse:[ kind == LLVMDoubleTypeKind ifTrue:[ 
        aStream nextPutAll:'double'  
    ] ifFalse:[ kind == LLVMX86_FP80TypeKind ifTrue:[ 
        aStream nextPutAll:'x86-fp80'  
    ] ifFalse:[ kind == LLVMFP128TypeKind ifTrue:[ 
        aStream nextPutAll:'fp128'  
    ] ifFalse:[ kind == LLVMLabelTypeKind ifTrue:[ 
        aStream nextPutAll:'label'  
    ] ifFalse:[ kind == LLVMIntegerTypeKind ifTrue:[ 
        aStream nextPutAll:'int'  
    ] ifFalse:[ kind == LLVMFunctionTypeKind ifTrue:[ 
        aStream nextPutAll:'funct'  
    ] ifFalse:[ kind == LLVMStructTypeKind ifTrue:[ 
        aStream nextPutAll:'struct'  
    ] ifFalse:[ kind == LLVMArrayTypeKind ifTrue:[ 
        aStream nextPutAll:'array'  
    ] ifFalse:[ kind == LLVMPointerTypeKind ifTrue:[ 
        aStream nextPutAll:'ptr'  
    ] ifFalse:[ kind == LLVMVectorTypeKind ifTrue:[ 
        aStream nextPutAll:'vector'  
    ] ifFalse:[ kind == LLVMMetadataTypeKind ifTrue:[ 
        aStream nextPutAll:'md'  
    ] ifFalse:[ kind == LLVMX86_MMXTypeKind ifTrue:[ 
        aStream nextPutAll:'x86-mmx'  
    ]]]]]]]]]]]]]]].
    aStream nextPutAll: ']'.

    "Created: / 11-07-2015 / 07:08:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

